import { RequestManager, SecurityTokenProvider, AxiosRequestManager, HateoasClient, SignalRSocketManager } from 'fancy-hateoas-client';
export { HateoasClient } from 'fancy-hateoas-client';
import * as i0 from '@angular/core';
import { inject, makeEnvironmentProviders, Directive } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { firstValueFrom, Subject } from 'rxjs';
import { ActivatedRoute } from '@angular/router';

/**
 * A special angular implementation for the RequestManager using angulars HttpClient.
 */
class AngularRequestManager extends RequestManager {
    constructor() {
        super(...arguments);
        this._httpClient = inject(HttpClient);
        this._tokenProvider = inject(SecurityTokenProvider, { optional: true });
    }
    async request(method, url, body) {
        let securityToken = null;
        if (this._tokenProvider) {
            securityToken = await this._tokenProvider.retrieveCurrentToken();
        }
        let headers = new HttpHeaders()
            .set('Content-Type', 'application/json')
            .set('X-Requested-With', 'XmlHttpRequest');
        if (securityToken) {
            headers = headers.set('Authorization', 'Bearer ' + securityToken);
        }
        if (method !== 'GET') {
            const xsrfToken = this.getXsrfCookie();
            if (xsrfToken) {
                headers = headers.set('X-XSRF-TOKEN', xsrfToken);
            }
        }
        try {
            let response = await firstValueFrom(this._httpClient.request(method, url, { body, headers }));
            return response;
        }
        catch (errorResponse) {
            if (errorResponse.status === 401) {
                // Need to redirect to sign in 
                const currentUrl = window.location.href;
                window.location.href = '/login?redirectUrl=' + encodeURIComponent(currentUrl);
            }
        }
    }
    getXsrfCookie(cookieName) {
        let name = cookieName || 'XSRF-TOKEN';
        const o = document.cookie.split(";").map(t => t.trim()).filter(t => t.startsWith(name + "="));
        return 0 === o.length ? null : decodeURIComponent(o[0].split("=")[1]);
    }
}

var HateoasClientFeatureKind;
(function (HateoasClientFeatureKind) {
    HateoasClientFeatureKind[HateoasClientFeatureKind["AxiosRequestManager"] = 0] = "AxiosRequestManager";
    HateoasClientFeatureKind[HateoasClientFeatureKind["AngularRequestManager"] = 1] = "AngularRequestManager";
})(HateoasClientFeatureKind || (HateoasClientFeatureKind = {}));
function withAxiosRequestManager() {
    return {
        kind: HateoasClientFeatureKind.AxiosRequestManager,
        providers: [
            {
                provide: RequestManager,
                useFactory: () => new AxiosRequestManager()
            }
        ]
    };
}
function withAngularRequestManager() {
    return {
        kind: HateoasClientFeatureKind.AngularRequestManager,
        providers: [
            {
                provide: RequestManager,
                useFactory: () => new AngularRequestManager()
            }
        ]
    };
}
function provideHateoasClient(...features) {
    // Validate features
    if (features?.length != 1) {
        throw new Error("You need to specifiy either the 'AxiosRequestManager' or the 'AngularRequestManager'");
    }
    return makeEnvironmentProviders([
        {
            provide: HateoasClient,
            useFactory: () => new HateoasClient(inject(RequestManager), new SignalRSocketManager())
        },
        // Add providers from services
        features?.map(f => f.providers)
    ]);
}

/**
 * Base class for angular components which represent a view and are backed by a hateoas resource.
 */
class TypedViewBase {
    constructor() {
        /**
         * A subject which gets triggered each time a new view model was loaded.
         */
        this._viewModelLoadedSubject = new Subject();
        /**
         * The activated route service.
         */
        this.activatedRoute = inject(ActivatedRoute);
        /**
         * The hateoas client.
         */
        this.hateoasClient = inject(HateoasClient);
        /**
         * A public observable to be used to subscribe to the view model loaded subject.
         */
        this.viewModelLoaded$ = this._viewModelLoadedSubject.asObservable();
        /**
         * The current view model.
         */
        this.viewModel = null;
        // Subscribe to current url and load corresponding view model
        this.activatedRoute.params.subscribe(params => {
            this.viewModelUrl = params['url'];
            this.reloadViewModel();
        });
    }
    /**
     * Reloads the current view model.
     */
    async reloadViewModel() {
        if (this.viewModelUrl) {
            this.viewModel = await this.hateoasClient.fetch(this.viewModelUrl);
            this.viewModelOnLoaded();
            this._viewModelLoadedSubject.next(this.viewModel);
            return this.viewModel;
        }
        else {
            return Promise.resolve({});
        }
    }
    /**
     * A life cycle hook to use when the implementing class needs to know when a new view model is availalbe.
     */
    viewModelOnLoaded() {
        // Empty default implementation
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.1.2", ngImport: i0, type: TypedViewBase, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.1.2", type: TypedViewBase, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.1.2", ngImport: i0, type: TypedViewBase, decorators: [{
            type: Directive
        }], ctorParameters: function () { return []; } });
/**
 * Base class for angular components which represent a untyped view and are backed by a hateoas resource.
 */
class ViewBase extends TypedViewBase {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.1.2", ngImport: i0, type: ViewBase, deps: null, target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.1.2", type: ViewBase, usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.1.2", ngImport: i0, type: ViewBase, decorators: [{
            type: Directive
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { HateoasClientFeatureKind, TypedViewBase, ViewBase, provideHateoasClient, withAngularRequestManager, withAxiosRequestManager };
//# sourceMappingURL=fancy-ngx-hateoas-client.mjs.map
